@@ -2,9 +2,7 @@
 import os
 import tempfile
 
-import nose
-from nose.tools import assert_true
-from nose.plugins.skip import SkipTest
+import pytest
 
 from xonsh.tools import ON_WINDOWS
 from xonsh.completers.man import complete_from_man
@@ -13,13 +11,13 @@ from tools import mock_xonsh_env
 
 _OLD_MANPATH = None
 
-def setup():
+def setup_module():
     global _OLD_MANPATH
     _OLD_MANPATH = os.environ.get('MANPATH', None)
     os.environ['MANPATH'] = os.path.dirname(os.path.abspath(__file__))
 
 
-def teardown():
+def teardown_module():
     global _OLD_MANPATH
     if _OLD_MANPATH is None:
         del os.environ['MANPATH']
@@ -27,15 +25,10 @@ def teardown():
         os.environ['MANPATH'] = _OLD_MANPATH
 
 
+@pytest.mark.skipif(ON_WINDOWS, reason='No man completions on Windows')
 def test_man_completion():
-    if ON_WINDOWS:
-        raise SkipTest
     with tempfile.TemporaryDirectory() as tempdir:
         with mock_xonsh_env({'XONSH_DATA_DIR': tempdir}):
             completions = complete_from_man('--', 'yes --', 4, 6, __xonsh_env__)
-        assert_true('--version' in completions)
-        assert_true('--help' in completions)
-
-
-if __name__ == '__main__':
-    nose.runmodule()
+        assert ('--version' in completions)
+        assert ('--help' in completions)
