@@ -9,25 +9,39 @@ import subprocess
 from collections import defaultdict
 from contextlib import contextmanager
 
-from nose.plugins.skip import SkipTest
+import pytest
 
 from xonsh.built_ins import ensure_list_of_strs
-builtins.__xonsh_env__ = {}
+from xonsh.environ import Env
 from xonsh.base_shell import BaseShell
+from xonsh.execer import Execer
+from xonsh.tools import XonshBlockError
 
 
 VER_3_4 = (3, 4)
 VER_3_5 = (3, 5)
 VER_MAJOR_MINOR = sys.version_info[:2]
 VER_FULL = sys.version_info[:3]
-ON_MAC = (platform.system() == 'Darwin')
+ON_DARWIN = (platform.system() == 'Darwin')
+ON_WINDOWS = (platform.system() == 'Windows')
+
+
+skip_if_py34 = pytest.mark.skipif(VER_MAJOR_MINOR < VER_3_5,
+                                   reason="Py3.5+ only test")
+
+skip_if_py35plus = pytest.mark.skipif(VER_MAJOR_MINOR < VER_3_5,
+                               reason="Py3.5+ only test")
+
+
 
 def sp(cmd):
     return subprocess.check_output(cmd, universal_newlines=True)
 
+
 class DummyStyler():
     styles = defaultdict(None.__class__)
 
+
 class DummyBaseShell(BaseShell):
 
     def __init__(self):
@@ -61,6 +75,7 @@ def mock_xonsh_env(xenv):
     builtins.__xonsh_subproc_captured__ = sp
     builtins.__xonsh_subproc_uncaptured__ = sp
     builtins.__xonsh_ensure_list_of_strs__ = ensure_list_of_strs
+    builtins.XonshBlockError = XonshBlockError
     builtins.evalx = eval
     builtins.execx = None
     builtins.compilex = None
@@ -78,21 +93,49 @@ def mock_xonsh_env(xenv):
     del builtins.__xonsh_subproc_captured__
     del builtins.__xonsh_subproc_uncaptured__
     del builtins.__xonsh_ensure_list_of_strs__
+    del builtins.XonshBlockError
     del builtins.evalx
     del builtins.execx
     del builtins.compilex
     del builtins.aliases
 
 
-def skipper():
-    """Raises SkipTest"""
-    raise SkipTest
+#
+# Execer tools
+#
+
+DEBUG_LEVEL = 0
+EXECER = None
+
+
+def execer_setup():
+    # only setup one parser
+    global EXECER
+    if EXECER is None:
+        EXECER = Execer(debug_level=DEBUG_LEVEL, login=False)
+
+
+def check_exec(input, **kwargs):
+    with mock_xonsh_env(None):
+        if not input.endswith('\n'):
+            input += '\n'
+        EXECER.debug_level = DEBUG_LEVEL
+        EXECER.exec(input, **kwargs)
+
+
+def check_eval(input):
+    env = Env({'AUTO_CD': False, 'XONSH_ENCODING': 'utf-8',
+               'XONSH_ENCODING_ERRORS': 'strict', 'PATH': []})
+    if ON_WINDOWS:
+        env['PATHEXT'] = ['.COM', '.EXE', '.BAT', '.CMD']
+    with mock_xonsh_env(env):
+        EXECER.debug_level = DEBUG_LEVEL
+        EXECER.eval(input)
+
+
+def check_parse(input):
+    with mock_xonsh_env(None):
+        EXECER.debug_level = DEBUG_LEVEL
+        tree = EXECER.parse(input, ctx=None)
+    return tree
 
-def skip_if(cond):
-    """Skips a test under a given condition."""
-    def dec(f):
-        if cond:
-            return skipper
-        else:
-            return f
-    return dec
