@@ -5,13 +5,14 @@ import sys
 import re
 import json
 import shlex
+import tempfile
 import builtins
 import subprocess
 from warnings import warn
 from functools import lru_cache
-from tempfile import NamedTemporaryFile
 from collections import MutableMapping, Mapping, Sequence
 
+from xonsh.lazyasd import LazyObject
 from xonsh.tools import to_bool, ensure_string
 from xonsh.platform import ON_WINDOWS, ON_CYGWIN
 
@@ -234,7 +235,7 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
     if not use_tmpfile:
         cmd.append(command)
     else:
-        tmpfile = NamedTemporaryFile(suffix=tmpfile_ext, delete=False)
+        tmpfile = tempfile.NamedTemporaryFile(suffix=tmpfile_ext, delete=False)
         tmpfile.write(command.encode('utf8'))
         tmpfile.close()
         cmd.append(tmpfile.name)
@@ -262,7 +263,12 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
     return env, aliases
 
 
-ENV_RE = re.compile('__XONSH_ENV_BEG__\n(.*)__XONSH_ENV_END__', flags=re.DOTALL)
+ENV_RE = LazyObject(lambda: re.compile('__XONSH_ENV_BEG__\n(.*)'
+                                       '__XONSH_ENV_END__', flags=re.DOTALL),
+                    globals(), 'ENV_RE')
+ENV_SPLIT_RE = LazyObject(lambda: re.compile('^([^=]+)=([^=]*|[^\n]*)$',
+                                             flags=re.DOTALL|re.MULTILINE),
+                          globals(), 'ENV_SPLIT_RE')
 
 
 def parse_env(s):
@@ -271,8 +277,8 @@ def parse_env(s):
     if m is None:
         return {}
     g1 = m.group(1)
-    items = [line.split('=', 1) for line in g1.splitlines() if '=' in line]
-    env = dict(items)
+    g1 = g1[:-1] if g1.endswith('\n') else g1
+    env = dict(ENV_SPLIT_RE.findall(g1))
     return env
 
 
@@ -461,8 +467,6 @@ def ensure_shell(shell):
     return shell
 
 
-DEFAULT_SHELLS = ({'shell': 'bash'},)
-
 def _get_shells(shells=None, config=None, issue_warning=True):
     if shells is not None and config is not None:
         raise RuntimeError('Only one of shells and config may be non-None.')
@@ -475,7 +479,7 @@ def _get_shells(shells=None, config=None, issue_warning=True):
         else:
             from xonsh.environ import load_static_config
             conf = load_static_config(env, config)
-        shells = conf.get('foreign_shells', DEFAULT_SHELLS)
+        shells = conf.get('foreign_shells', ())
     return shells
 
 
