@@ -2,10 +2,9 @@
 """Implements a lazy JSON file class that wraps around json data."""
 import io
 import json
-from collections import Mapping, Sequence
-from contextlib import contextmanager
 import weakref
-
+import contextlib
+import collections.abc as abc
 
 
 def _to_json_with_size(obj, offset=0, sort_keys=False):
@@ -13,7 +12,7 @@ def _to_json_with_size(obj, offset=0, sort_keys=False):
         s = json.dumps(obj)
         o = offset
         n = size = len(s.encode())  # size in bytes
-    elif isinstance(obj, Mapping):
+    elif isinstance(obj, abc.Mapping):
         s = '{'
         j = offset + 1
         o = {}
@@ -36,7 +35,7 @@ def _to_json_with_size(obj, offset=0, sort_keys=False):
         n = len(s)
         o['__total__'] = offset
         size['__total__'] = n
-    elif isinstance(obj, Sequence):
+    elif isinstance(obj, abc.Sequence):
         s = '['
         j = offset + 1
         o = []
@@ -90,13 +89,13 @@ def dumps(obj, sort_keys=False):
     return s
 
 
-def dump(obj, fp, sort_keys=False):
+def ljdump(obj, fp, sort_keys=False):
     """Dumps an object to JSON file."""
     s = dumps(obj, sort_keys=sort_keys)
     fp.write(s)
 
 
-class Node(Mapping, Sequence):
+class LJNode(abc.Mapping, abc.Sequence):
     """A proxy node for JSON nodes. Acts as both sequence and mapping."""
 
     def __init__(self, offsets, sizes, root):
@@ -112,8 +111,8 @@ class Node(Mapping, Sequence):
         self.offsets = offsets
         self.sizes = sizes
         self.root = root
-        self.is_mapping = isinstance(self.offsets, Mapping)
-        self.is_sequence = isinstance(self.offsets, Sequence)
+        self.is_mapping = isinstance(self.offsets, abc.Mapping)
+        self.is_sequence = isinstance(self.offsets, abc.Sequence)
 
     def __len__(self):
         # recall that for maps, the '__total__' key is added and for
@@ -139,8 +138,8 @@ class Node(Mapping, Sequence):
                 f.seek(self.root.dloc + offset)
                 s = f.read(size)
             val = json.loads(s)
-        elif isinstance(offset, (Mapping, Sequence)):
-            val = Node(offset, size, self.root)
+        elif isinstance(offset, (abc.Mapping, abc.Sequence)):
+            val = LJNode(offset, size, self.root)
         else:
             raise TypeError('incorrect types for offset node')
         return val
@@ -187,7 +186,7 @@ class Node(Mapping, Sequence):
             raise NotImplementedError
 
 
-class LazyJSON(Node):
+class LazyJSON(LJNode):
     """Represents a lazy json file. Can be used like a normal Python
     dict or list.
     """
@@ -206,8 +205,8 @@ class LazyJSON(Node):
             self._f = open(f, 'r', newline='\n')
         self._load_index()
         self.root = weakref.proxy(self)
-        self.is_mapping = isinstance(self.offsets, Mapping)
-        self.is_sequence = isinstance(self.offsets, Sequence)
+        self.is_mapping = isinstance(self.offsets, abc.Mapping)
+        self.is_sequence = isinstance(self.offsets, abc.Sequence)
 
     def __del__(self):
         self.close()
@@ -217,7 +216,7 @@ class LazyJSON(Node):
         if not self.reopen and isinstance(self._f, io.IOBase):
             self._f.close()
 
-    @contextmanager
+    @contextlib.contextmanager
     def _open(self, *args, **kwargs):
         if self.reopen and isinstance(self._f, str):
             f = open(self._f, *args, **kwargs)
