@@ -2,8 +2,12 @@
 """Key bindings for prompt_toolkit xonsh shell."""
 import builtins
 
-from prompt_toolkit.filters import Filter, IsMultiline
+from prompt_toolkit.enums import DEFAULT_BUFFER
+from prompt_toolkit.filters import (Condition, Filter, IsMultiline,
+                                    HasSelection, EmacsInsertMode,
+                                    ViInsertMode)
 from prompt_toolkit.keys import Keys
+from xonsh.aliases import xonsh_exit
 from xonsh.tools import ON_WINDOWS
 
 env = builtins.__xonsh_env__
@@ -93,6 +97,16 @@ class EndOfLine(Filter):
 
         return bool(at_end and not last_line)
 
+
+# Copied from prompt-toolkit's key_binding/bindings/basic.py
+@Condition
+def ctrl_d_condition(cli):
+    """ Ctrl-D binding is only active when the default buffer is selected
+    and empty. """
+    return (cli.current_buffer_name == DEFAULT_BUFFER and
+            not cli.current_buffer.text)
+
+
 def can_compile(src):
     """Returns whether the code can be compiled, i.e. it is valid xonsh."""
     src = src if src.endswith('\n') else src + '\n'
@@ -113,6 +127,8 @@ def load_xonsh_bindings(key_bindings_manager):
     Load custom key bindings.
     """
     handle = key_bindings_manager.registry.add_binding
+    has_selection = HasSelection()
+    insert_mode = ViInsertMode() | EmacsInsertMode()
 
     @handle(Keys.Tab, filter=TabShouldInsertIndentFilter())
     def _(event):
@@ -122,11 +138,23 @@ def load_xonsh_bindings(key_bindings_manager):
         """
         event.cli.current_buffer.insert_text(env.get('INDENT'))
 
+    @handle(Keys.ControlX, Keys.ControlE, filter=~has_selection)
+    def open_editor(event):
+        """ Open current buffer in editor """
+        event.current_buffer.open_in_editor(event.cli)
+
     @handle(Keys.BackTab)
     def insert_literal_tab(event):
         """ Insert literal tab on Shift+Tab instead of autocompleting """
         event.cli.current_buffer.insert_text(env.get('INDENT'))
 
+    @handle(Keys.ControlD, filter=ctrl_d_condition)
+    def call_exit_alias(event):
+        """Use xonsh exit function"""
+        b = event.cli.current_buffer
+        b.accept_action.validate_and_handle(event.cli, b)
+        xonsh_exit([])
+
     @handle(Keys.ControlJ, filter=IsMultiline())
     def multiline_carriage_return(event):
         """ Wrapper around carriage_return multiline parser """
@@ -149,5 +177,32 @@ def load_xonsh_bindings(key_bindings_manager):
         b.cursor_left(count=abs(relative_begin_index))
         b.cursor_down(count=1)
 
+
+    @handle(Keys.ControlI, filter=insert_mode)
+    def generate_completions(event):
+        """
+        Tab-completion: where the first tab completes the common suffix and the
+        second tab lists all the completions.
+
+        Notes
+        -----
+        This method was forked from the mainline prompt-toolkit repo.
+        Copyright (c) 2014, Jonathan Slenders, All rights reserved.
+        """
+        b = event.current_buffer
+
+        def second_tab():
+            if b.complete_state:
+                b.complete_next()
+            else:
+                event.cli.start_completion(select_first=False)
+
+        # On the second tab-press, or when already navigating through
+        # completions.
+        if event.is_repeat or b.complete_state:
+            second_tab()
+        else:
+            event.cli.start_completion(insert_common_part=True,
+                                    select_first=False)
 def _is_blank(l):
     return len(l.strip()) == 0
